Node('fs.psi.ch',
     'small vacuum furnace',
     'tcp://5000',
)

Mod('T',
    'frappy_psi.picontrol.PI2',
    'controlled Temperature on sample (2nd loop)',
    input = 'T_sample',
    output = 'T_reg',
    relais = 'relais',
    p = 1.2,
    i = 0.005,
   )

Mod('T_reg',
    'frappy_psi.picontrol.PI',
    'controlled Temperature on heater',
    input = 'T_htr',
    output = 't_out',
    relais = 'relais',
    p = 1,
    i = 0.003,
   )

Mod('p_reg',
    'frappy_psi.picontrol.PI',
    'controlled pressure',
    input = 'p',
    output = 'p_out',
    relais = 'relais',
    p = 1,
    i = 0.005,
   )

Mod('T_htr',
    'frappy_psi.ionopimax.CurrentInput',
    'heater temperature',
    addr = 'ai4',
    valuerange = (0, 1372),
    value = Param(unit='degC'),
    
   )


Mod('T_sample',
    'frappy_psi.ionopimax.CurrentInput',
    'sample temperature',
    addr = 'ai3',
    valuerange = (0, 1372),
    value = Param(unit='degC'),

   )

Mod('T_extra',
    'frappy_psi.ionopimax.CurrentInput',
    'extra temperature',
    addr = 'ai2',
    valuerange = (0, 1372),
    value = Param(unit='degC'),

   )


Mod('T_wall',
    'frappy_psi.ionopimax.VoltageInput',
    'furnace wall temperature',
    addr = 'av2',
    rawrange = (0, 1.5),
    valuerange = (0, 150),
    value = Param(unit='degC'),
   )

Mod('htr_io',
    'frappy_psi.bkpower.IO',
    'powersupply communicator',
    uri = 'serial:///dev/ttyUSBupper',
   )

Mod('htr',
    'frappy_psi.bkpower.Power',
    'heater power',
    io= 'htr_io',
   )

Mod('t_out',
    'frappy_psi.bkpower.Output',
    'heater output',
    p_value = 'p_out',
    io = 'htr_io',
    maxvolt = 50,
    maxcurrent = 2,
   )

Mod('relais',
    'frappy_psi.ionopimax.DigitalOutput',
    'relais for power output',
    addr = 'o2',
   )

Mod('interlocks',
    'frappy_psi.furnace.Interlocks',
    'interlock parameters',
    input = 'T_htr',
    wall_T = 'T_wall',
    htr_T = 'T_htr',
    main_T = 'T_sample',
    extra_T = 'T_extra',
    vacuum = 'p',
    relais = 'relais',
    control = 'T',
    wall_limit = 100,
    vacuum_limit = 0.1,
   )

Mod('p',
    'frappy_psi.ionopimax.LogVoltageInput',
    'pressure reading',
    addr = 'av1',
    rawrange = (1.82, 8.6),
    valuerange = (5e-9, 1000),
    value = Param(unit='mbar'),
   )

Mod('vso',
    'frappy_psi.ionopimax.VoltagePower',
    'voltage power output',
    target = 24,
    export = False,
   )
